@@ -7,7 +7,7 @@ use Socket;
 use Carp qw(carp croak);
 use vars qw($VERSION);
 
-$VERSION = '1.04';
+$VERSION = '1.10';
 
 sub spawn {
   my $package = shift;
@@ -20,13 +20,15 @@ sub spawn {
   $self->{_prefix} .= '_' unless $self->{_prefix} =~ /\_$/;
   $self->{session_id} = POE::Session->create(
 	object_states => [
-	   $self => { shutdown       => '_shutdown',
-		      send_event     => '__send_event',
-		      send_to_client => '_send_to_client',
-		      disconnect     => '_disconnect',
-		      terminate      => '_terminate',
+	   $self => { shutdown      => '_shutdown',
+		      send_event          => '__send_event',
+		      send_to_client      => '_send_to_client',
+		      send_to_all_clients => '_send_to_all_clients',
+		      disconnect          => '_disconnect',
+		      terminate           => '_terminate',
+          start_listener      => '_start_listener',
 	            },
-	   $self => [ qw(_start register unregister _accept_client _accept_failed _conn_input _conn_error _conn_flushed _conn_alarm _send_to_client __send_event _disconnect) ],
+	   $self => [ qw(_start register unregister _accept_client _accept_failed _conn_input _conn_error _conn_flushed _conn_alarm _send_to_client __send_event _disconnect _send_to_all_clients) ],
 	],
 	heap => $self,
 	( ref($options) eq 'HASH' ? ( options => $options ) : () ),
@@ -88,6 +90,19 @@ sub _start {
     $kernel->detach_myself();
   }
 
+  $kernel->call( $self->{session_id}, 'start_listener' );
+  return;
+}
+
+sub start_listener {
+  my $self = shift;
+  $poe_kernel->call( $self->{session_id}, 'start_listener', @_ );
+}
+
+sub _start_listener {
+  my ($kernel,$self) = @_[KERNEL,OBJECT];
+  return if $self->{listener};
+
   $self->{listener} = POE::Wheel::SocketFactory->new(
       ( defined $self->{address} ? ( BindAddress => $self->{address} ) : () ),
       ( defined $self->{port} ? ( BindPort => $self->{port} ) : ( BindPort => 0 ) ),
@@ -147,6 +162,13 @@ sub client_info {
   return \%hash;
 }
 
+sub client_wheel {
+  my $self = shift;
+  my $id = shift || return;
+  return unless $self->_conn_exists( $id );
+  return $self->{clients}->{ $id }->{wheel};
+}
+
 sub _get_filters {
     my ($client_filter, $client_infilter, $client_outfilter) = @_;
     if (defined $client_infilter or defined $client_outfilter) {
@@ -215,7 +237,7 @@ sub _test_filter {
 sub _accept_failed {
   my ($kernel,$self,$operation,$errnum,$errstr,$wheel_id) = @_[KERNEL,OBJECT,ARG0..ARG3];
   warn "Wheel $wheel_id generated $operation error $errnum: $errstr\n";
-  delete $self->{listener};
+  delete $self->{listener} if $operation eq 'listen';
   $self->_send_event( $self->{_prefix} . 'listener_failed', $operation, $errnum, $errstr );
   return;
 }
@@ -407,8 +429,9 @@ sub _send_to_client {
   return unless $output;
 
   if ( ref $output eq 'ARRAY' ) {
-    my $first = shift @{ $output };
-    $self->{clients}->{ $id }->{BUFFER} = $output if scalar @{ $output };
+    my $temp = [ @{ $output } ];
+    my $first = shift @{ $temp };
+    $self->{clients}->{ $id }->{BUFFER} = $temp if scalar @{ $temp };
     $self->{clients}->{ $id }->{wheel}->put($first) if defined $first;
     return 1;
   }
@@ -417,6 +440,19 @@ sub _send_to_client {
   return 1;
 }
 
+sub send_to_all_clients {
+  my $self = shift;
+  $poe_kernel->call( $self->{session_id}, '_send_to_all_clients', @_ );
+}
+
+sub _send_to_all_clients {
+  my ($kernel,$self,$output) = @_[KERNEL,OBJECT,ARG0];
+  return unless $output;
+  $self->send_to_client( $_, $output ) for
+    keys %{ $self->{clients} };
+  return 1;
+}
+
 q{Putting the test into POE};
 
 __END__
@@ -664,6 +700,12 @@ Send some output to a connected client. First parameter must be a valid client i
 The second parameter may also be an arrayref of items to send to the client. If the filter you have used requires an arrayref as
 input, nest that arrayref within another arrayref.
 
+=item C<send_to_all_clients>
+
+Send some output to all connected clients. The parameter is a string of text to send.
+The parameter may also be an arrayref of items to send to the clients. If the filter you have used requires an arrayref as
+input, nest that arrayref within another arrayref.
+
 =item C<client_info>
 
 Retrieve socket information of a given client. Requires a valid client ID as a parameter. If called in a list context it returns a list 
@@ -675,6 +717,11 @@ with the following keys:
   'sockaddr', our address;
   'sockport', our TCP port;
 
+=item C<client_wheel>
+
+Retrieve the L<POE::Wheel::ReadWrite> object of a given client. Requires a valid client ID as a parameter. This enables one to 
+manipulate the given L<POE::Wheel::ReadWrite> object, say to switch L<POE::Filter>.
+
 =item C<disconnect>
 
 Places a client connection in pending disconnect state. Requires a valid client ID as a parameter. Set this, then send an applicable message to the client using send_to_client() and the client connection will be terminated.
@@ -699,6 +746,10 @@ Access to the L<POE::Wheel::SocketFactory> method of the underlying listening so
 
 Returns the port that the component is listening on.
 
+=item C<start_listener>
+
+If the listener fails on C<listen> you can attempt to restart it with this.
+
 =back
 
 =head1 INPUT EVENTS
@@ -723,8 +774,16 @@ Terminates the component. Shuts down the listener and disconnects connected clie
 
 =item C<send_to_client>
 
-Send some output to a connected client. First parameter must be a valid client ID. 
-Second parameter is a string of text to send.
+Send some output to a connected client. First parameter must be a valid client id. Second parameter is a string of text to send.
+The second parameter may also be an arrayref of items to send to the client. If the filter you have used requires an arrayref as
+input, nest that arrayref within another arrayref.
+
+
+=item C<send_to_all_clients>
+
+Send some output to all connected clients. The parameter is a string of text to send.
+The parameter may also be an arrayref of items to send to the clients. If the filter you have used requires an arrayref as
+input, nest that arrayref within another arrayref.
 
 =item C<disconnect>
 
@@ -734,6 +793,10 @@ Places a client connection in pending disconnect state. Requires a valid client
 
 Immediately disconnects a client conenction. Requires a valid client ID as a parameter.
 
+=item C<start_listener>
+
+If the listener fails on C<listen> you can attempt to restart it with this.
+
 =back
 
 =head1 OUTPUT EVENTS
@@ -753,6 +816,9 @@ Generated if the component cannot either start a listener or there is a problem
 accepting client connections. ARG0 contains the name of the operation that failed. 
 ARG1 and ARG2 hold numeric and string values for $!, respectively.
 
+If the operation was C<listen>, the component will remove the listener.
+You may attempt to start it again using C<start_listener>.
+
 =item C<testd_connected>
 
 Generated whenever a client connects to the component. ARG0 is the client ID, ARG1
